# Machine Learning with Python
"""
Machine Learning (ML) with Python is about programming computers to improve their performance at tasks through learning from data, without being explicitly programmed for the task. In Python, ML is facilitated by libraries such as scikit-learn, TensorFlow, and PyTorch. These libraries come with pre-built algorithms for tasks like classification, regression, clustering, and neural networks, making it easier to implement ML solutions. The process generally involves collecting data, preprocessing it, splitting it into training and test sets, choosing an appropriate model, training the model on the training data, and evaluating its performance with the test data.
"""

# Example: # Example 1: Simple linear regression using scikit-learn
import numpy as np
from sklearn.linear_model import LinearRegression
# Training data
X = np.array([[1], [2], [3], [4]]) # features
y = np.dot(X, np.array([2])) + 3 # labels
# Model initialization and fitting
model = LinearRegression()
model.fit(X, y)
# Making predictions
predictions = model.predict(X)


# Example: # Example 2: Train-test split
from sklearn.model_selection import train_test_split
X = [[0, 0], [1, 1], [2, 2], [3, 3]]
y = [0, 1, 2, 3]
# Split arrays or matrices into random train and test subsets
X_train, X_test, y_train, y_test = train_test_split(X, y, test_size=0.25)


# Example: # Example 3: Image classification with a neural network using TensorFlow
import tensorflow as tf
from tensorflow.keras.layers import Dense, Flatten, Conv2D
from tensorflow.keras import Model
# Define a simple convolutional neural network model
model = tf.keras.models.Sequential([
  Conv2D(32, kernel_size=(3, 3), activation='relu', input_shape=(28, 28, 1)),
  Flatten(),
  Dense(128, activation='relu'),
  Dense(10)
])
# Compile the model
model.compile(optimizer='adam',
              loss=tf.keras.losses.SparseCategoricalCrossentropy(from_logits=True),
              metrics=['accuracy'])
# Assume we have training and testing data loaded in train_images, train_labels, test_images, and test_labels
model.fit(train_images, train_labels, epochs=5)
model.evaluate(test_images, test_labels, verbose=2)


# Exercise: 1. Use scikit-learn's Iris dataset to build a classifier that can predict the species of an Iris based on sepal length, sepal width, petal length, and petal width. Try out at least two different classifiers and compare their performance.
# Exercise: 2. Using the Boston housing dataset, available in scikit-learn, create a regression model that predicts the price of a house based on various features. Evaluate your model's performance using mean squared error.
# Exercise: 3. Explore unsupervised learning by clustering a dataset (for example, customer data for market segmentation) using k-means clustering in scikit-learn. Determine the optimal number of clusters using the elbow method.
# Exercise: 4. Create your own neural network with TensorFlow to recognize handwritten digits from the MNIST dataset. Experiment with different network architectures and activation functions.
# Exercise: 5. Implement a text classification model using PyTorch that classifies movie reviews as positive or negative. Preprocess the text data, convert it into tensors, and use an RNN (Recurrent Neural Network) for the classification task.
# Additional Resource: Scikit-learn Official Documentation: https://scikit-learn.org/stable/documentation.html
# Additional Resource: TensorFlow Tutorials: https://www.tensorflow.org/tutorials
# Additional Resource: PyTorch Tutorials: https://pytorch.org/tutorials
# Additional Resource: Hands-On Machine Learning with Scikit-Learn, Keras, and TensorFlow (Book by Aurlien Gron)
# Additional Resource: Python Machine Learning (Book by Sebastian Raschka and Vahid Mirjalili)