# Functions and Modules
"""
In Python, a function is a block of organized, reusable code that is used to perform a single, related action. Functions provide better modularity for your application and a high degree of code reusing. As you already know, Python gives you many built-in functions like print(), etc. but you can also create your own functions. Functions are defined using the 'def' keyword. On the other hand, modules are files consisting of Python code. A module can define functions, classes, and variables. A module can also include runnable code. Grouping related code into a module makes the code easier to understand and use. You can import modules using the 'import' keyword.
"""

# Example: # Example of a simple function

 def greet(name):
    return "Hello, " + name + "!"

# Calling the function
print(greet('Alice'))


# Example: # Example of a function with default parameters

 def calculate_area(radius, pi=3.14):
    return pi * (radius ** 2)

# Calling the function with default pi
print(calculate_area(5))

# Calling the function with custom pi value
print(calculate_area(5, pi=3.14159))


# Example: # Example of importing a module

 import math

# Use of a function from the math module
print(math.sqrt(16))


# Example: # Example of creating a module

# Create a file called 'math_utils.py' with the following content:
# def add(a, b):
#     return a + b
#
# def subtract(a, b):
#     return a - b

# Importing the custom module in another Python file
from math_utils import add, subtract

# Using the imported functions
print(add(4, 5))
print(subtract(10, 5))


# Exercise: Write a function 'max_of_three' that takes three numbers as arguments and returns the largest of them.
# Exercise: Create a function 'is_vowel' that takes a character (i.e. a string of length 1) and returns True if it is a vowel, False otherwise.
# Exercise: Define a function 'sum_all' that takes any number of integer arguments and returns their sum.
# Exercise: Create a module 'string_utils' with the following functions: 'count_vowels' which counts the number of vowels in a string, 'is_palindrome' which checks whether a string is a palindrome or not. Make sure to use proper docstrings to document your module.
# Exercise: Write a function 'build_profile' that accepts a first name, a last name, and an arbitrary number of keyword arguments 'info'. The function should return a dictionary with the name and all the information passed through keyword arguments.
# Additional Resource: Python Functions - https://docs.python.org/3/tutorial/controlflow.html#defining-functions
# Additional Resource: Modules - https://docs.python.org/3/tutorial/modules.html
# Additional Resource: The Python Standard Library - https://docs.python.org/3/library/
# Additional Resource: Real Python: Defining Your Own Python Function - https://realpython.com/defining-your-own-python-function/
# Additional Resource: Real Python: Absolute vs Relative Imports in Python - https://realpython.com/absolute-vs-relative-python-imports/