# Object-Oriented Programming
"""
Object-Oriented Programming (OOP) is a programming paradigm based on the concept of 'objects', which can contain data, in the form of fields (often known as attributes or properties), and code, in the form of procedures (often known as methods). In OOP, computer programs are designed by making them out of objects that interact with one another. There are four basic principles of object-oriented programming: Encapsulation, Abstraction, Inheritance, and Polymorphism.

Encapsulation means that a group of related properties, methods, and other members are treated as a single unit or object. Abstraction involves the hiding of unnecessary details from the users. Inheritance is a way to form new classes using classes that have already been defined. Polymorphism allows for using a class exactly like its parent so there is no confusion with mixing types.

This approach to programming is significant because it helps in structuring software in a way that is easier to maintain and also reuse.
"""

# Example: # Example 1: Defining a class

class Car:
    def __init__(self, brand, model, year):
        self.brand = brand
        self.model = model
        self.year = year

    def display_info(self):
        print(f'This car is a {self.year} {self.brand} {self.model}')

# Creating an instance of Car
my_car = Car('Toyota', 'Corolla', 2020)
my_car.display_info()  # Output: This car is a 2020 Toyota Corolla
# Example: # Example 2: Encapsulation

class Computer:
    def __init__(self):
        self.__max_price = 900

    def sell(self):
        print(f'Selling Price: {self.__max_price}')

    def set_max_price(self, price):
        self.__max_price = price

# Using encapsulation
c = Computer()
c.sell()  # Output: Selling Price: 900

c.__max_price = 1000  # Won't change price because it is private

c.sell()  # Output: Selling Price: 900

c.set_max_price(1000)
c.sell()  # Output: Selling Price: 1000
# Example: # Example 3: Inheritance

class Vehicle:
    def __init__(self, brand, model):
        self.brand = brand
        self.model = model

    def display_vehicle_info(self):
        print(f'Vehicle Info: {self.brand} {self.model}')

# Derived class

class Car(Vehicle):
    def __init__(self, brand, model, year):
        super().__init__(brand, model)  # Call the super class constructor
        self.year = year

# Using Inheritance
car = Car('Ford', 'Mustang', 2020)
car.display_vehicle_info() # Output: Vehicle Info: Ford Mustang
# Example: # Example 4: Polymorphism

class Animal:
    def speak(self):
        pass

class Dog(Animal):
    def speak(self):
        return 'Woof!'

class Cat(Animal):
    def speak(self):
        return 'Meow!'

# Using Polymorphism
def animal_sound(animal):
    print(animal.speak())

animal_sound(Dog())  # Output: Woof!
animal_sound(Cat())  # Output: Meow!
# Exercise: 1. Define a class named 'Rectangle'. It should have attributes for 'width' and 'height'. It should also have a method that calculates its area.
# Exercise: 2. Create a class 'Circle' with an attribute 'radius' and a method to compute its area. Then create a subclass 'Cylinder' derived from 'Circle' with an additional attribute 'height' and a method to calculate its volume.
# Exercise: 3. Define a class 'Person' with properties 'name' and 'age', and a method 'introduce_self' that prints a greeting. Then create two instances of the 'Person' class and call their 'introduce_self' methods.
# Exercise: 4. Implement a class 'Book' with four attributes: 'title', 'author', 'genre', 'year'. Include a method that displays the complete info of the Book instance. Then, create a few instances of books and invoke the display method.
# Exercise: 5. Write a Python class to find the three elements that sum to zero from a list of n integers. Implement a method inside the class for finding and displaying those elements.
# Additional Resource: https://docs.python.org/3/tutorial/classes.html
# Additional Resource: https://realpython.com/python3-object-oriented-programming/
# Additional Resource: https://www.learnpython.org/en/Object-Oriented_Programming
# Additional Resource: http://python-textbok.readthedocs.io/en/1.0/Object_Oriented_Programming.html
# Additional Resource: https://www.programiz.com/python-programming/object-oriented-programming