# File Handling
"""
In Python, file handling is a way of storing, accessing, and manipulating data in a file. It is an essential part of any real-world application. Python provides several functions and methods to handle files, which includes opening a file, reading or writing to it, and closing it when you are done. There are different modes for opening a file (e.g., read, write, append) depending on the action you want to perform. It is important to always close the file to free up system resources. Python also provides the 'with' statement to help ensure that files are closed properly even if an error occurs during file operations.
"""

# Example: # Example 1: Opening and reading a file
with open('example.txt', 'r') as file:
    data = file.read()
    print(data)
# Example: # Example 2: Writing to a file, overwriting if exists
with open('example.txt', 'w') as file:
    file.write('Hello, World!')
# Example: # Example 3: Appending to a file (adds to the end)
with open('example.txt', 'a') as file:
    file.write('\nAnother line!')
# Example: # Example 4: Reading a file line by line
with open('example.txt', 'r') as file:
    for line in file:
        print(line.strip()) # strip() removes newline characters
# Example: # Example 5: Using file.tell() and file.seek() methods
text_file = open('example.txt', 'r')
initial_position = text_file.tell()
print('Current file position:', initial_position)
text_file.seek(10) # move the cursor to the 10th byte in the file
print('New file position:', text_file.tell())
text_file.close()
# Exercise: 1. Write a Python program to read an entire text file.
# Exercise: 2. Write a Python program to append text to an existing file and display the entire file.
# Exercise: 3. Write a Python program to read the first n lines of a file.
# Exercise: 4. Write a Python program to read last n lines of a file.
# Exercise: 5. Write a Python function that takes filename and a number 'n' as input and returns the 'n' largest words from the file.
# Additional Resource: https://docs.python.org/3/tutorial/inputoutput.html#reading-and-writing-files
# Additional Resource: https://realpython.com/read-write-files-python/
# Additional Resource: https://www.programiz.com/python-programming/file-operation
# Additional Resource: https://www.w3schools.com/python/python_file_handling.asp
# Additional Resource: https://www.geeksforgeeks.org/file-handling-python/