# Advanced Topics (Decorators, Generators)
"""
In Python, decorators are functions that modify the behavior of another function. They allow you to 'decorate' a function without changing its code. A common use of decorators is extending the behavior of functions from external libraries (where you don't want to change the source code) or for debugging purposes. On the other hand, Generators are a simple way to create iterators. A generator is a function that returns an object on which you can call next() until it raises a StopIteration exception, signaling that all values have been generated. Generators use the 'yield' keyword instead of 'return' to return values.
"""

# Example: # Example of decorator

# Example: def my_decorator(func):

# Example:     def wrapper():

# Example:         print('Something is happening before the function is called.')

# Example:         func()

# Example:         print('Something is happening after the function is called.')

# Example:     return wrapper


# Example: @my_decorator

# Example: def say_hello():

# Example:     print('Hello!')


# Example: say_hello()

# Example: # Example of generator

# Example: def my_generator(x):

# Example:     i = 0

# Example:     while i < x:

# Example:         yield i

# Example:         i += 1


# Example: for value in my_generator(3):

# Example:     print(value)

# Exercise: 1. Write a decorator 'debug' that prints 'Executing:' followed by the name of the function and its arguments before it is executed.
# Exercise: 2. Create a generator function 'fibonacci' that generates the Fibonacci sequence indefinitely.
# Exercise: 3. Write a decorator 'authenticate' that only runs the decorated function if a provided password argument equals 'secret'.
# Exercise: 4. Implement a generator that yields the square of numbers starting from 1 to a number n provided as an argument.
# Exercise: 5. Decorate a function with a Python built-in decorator and explain how it modifies the function's behavior.
# Additional Resource: {'title': 'PEP 318 -- Decorators for Functions and Methods', 'url': 'https://www.python.org/dev/peps/pep-0318/'}
# Additional Resource: {'title': 'PEP 255 -- Simple Generators', 'url': 'https://www.python.org/dev/peps/pep-0255/'}
# Additional Resource: {'title': 'Real Python - Python Decorators: A Step-by-Step Introduction', 'url': 'https://realpython.com/primer-on-python-decorators/'}
# Additional Resource: {'title': 'Real Python - Python Generators: An Introduction', 'url': 'https://realpython.com/introduction-to-python-generators/'}
# Additional Resource: {'title': 'Python Decorators Overview', 'url': 'https://www.thecodeship.com/patterns/guide-to-python-function-decorators/'}