# Functions and Modules
"""
In Python, functions are reusable blocks of code designed to perform a specific task. They help in dividing the programs into smaller, modular chunks. As your program grows more extensive and complex, functions make it more organized and manageable. They prevent redundancy and make the code more readable. A module, on the other hand, is a file containing Python definitions and statements. The file name is the module name with the suffix '.py' appended. Modules can define functions, classes, and variables that can be reused in other Python programs.
"""

# Example: # Example 1: Defining and calling a simple function

def greet(name):
    print(f'Hello, {name}!')

greet('Alice')
# This will output: Hello, Alice!

# Example: # Example 2: Function with a return value

def add(a, b):
    return a + b

result = add(5, 3)
print(result)
# This will output: 8

# Example: # Example 3: A module containing a variable, a function, and a class.
# mymodule.py

PI = 3.14159

def area(radius):
    return PI * (radius ** 2)

class Circle:
    def __init__(self, radius):
        self.radius = radius

    def compute_area(self):
        return PI * (self.radius ** 2)

# Example: # Example 4: Importing and using a module

# Assume 'mymodule.py' contains the code from Example 3
import mymodule

print(mymodule.PI)
# Outputs the value of PI

area_of_circle = mymodule.area(5)
print(area_of_circle)
# This will output the area of a circle with radius 5

my_circle = mymodule.Circle(3)
print(my_circle.compute_area())
# Outputs the area of the circle instance

# Exercise: # Exercise 1: Create a function called 'max_of_three' that takes three numbers as arguments and returns the largest of them.
# Exercise: # Exercise 2: Write a function 'is_prime' that takes a number and returns True if the number is a prime number, else returns False.
# Exercise: # Exercise 3: Create a module named 'calculator' with functions for addition, subtraction, multiplication, and division. Ensure that the division function handles the division by zero case.
# Exercise: # Exercise 4: Import the 'calculator' module from Exercise 3 in another file and perform various arithmetic operations.
# Exercise: # Exercise 5: Write a function 'factorial' that takes a number and returns its factorial. Use recursion to write this function.
# Additional Resource: Automate the Boring Stuff with Python - Al Sweigart (Chapter 3: Functions)
# Additional Resource: Python 3 Module of the Week - Doug Hellmann (https://pymotw.com/3/)
# Additional Resource: Python Documentation on Modules: https://docs.python.org/3/tutorial/modules.html
# Additional Resource: Learn Python the Hard Way - Zed Shaw (Exercise 19: Functions and Variables, Exercise 21: Functions Can Return Something)