# Exception Handling
"""
Exception handling in Python is a mechanism that allows a programmer to respond to error conditions (exceptions) that might arise during the execution of a program and disrupt its normal flow. An exception is a Python object that represents an error. When a Python script encounters a situation that it cannot cope with, it raises an exception. If the program does not handle the exception, the program will crash. To handle exceptions, Python uses try-except blocks. The try block lets you test a block of code for errors. The except block lets you handle the error. You can also use the 'else' clause to execute code if no exceptions were raised, and the 'finally' clause lets you execute code, regardless of the result of the try- and except blocks.
"""

# Example: # Example 1: Try-except block
try:
    result = 10 / 0
except ZeroDivisionError:
    print('You cannot divide by zero!')

# Example: # Example 2: Multiple except blocks
try:
    result = '10' + 10
except TypeError:
    print('Type Error: You cannot add a string and an integer.')
except ValueError:
    print('Value Error encountered!')

# Example: # Example 3: Else clause
try:
    result = 10 / 5
except ZeroDivisionError:
    print('You cannot divide by zero!')
else:
    print('Division successful:', result)

# Example: # Example 4: Finally clause
try:
    result = 10 / 0
except ZeroDivisionError:
    print('You cannot divide by zero!')
finally:
    print('This is executed regardless of exception')

# Example: # Example 5: Raising an exception
try:
    age = int(input('Enter your age: '))
    if age < 0:
        raise ValueError('Age cannot be negative.')
except ValueError as ve:
    print(ve)

# Exercise: 1. Write a program that attempts to open a file and prints out the file contents. Use a try-except block to catch the FileNotFoundError and print a user-friendly message.
# Exercise: 2. Create a function called 'divide' that takes two parameters (x and y) and returns the result of x divided by y. Use exception handling to catch any potential exceptions.
# Exercise: 3. Modify the 'divide' function from the previous exercise to include an else clause that prints out 'The division was successful!' when no exceptions are raised.
# Exercise: 4. Add a finally block to the 'divide' function that prints out 'Executing finally clause'.
# Exercise: 5. Write a function that takes user input and converts it to an integer. Use exception handling to catch and print messages for at least two different exception types.
# Additional Resource: https://docs.python.org/3/tutorial/errors.html
# Additional Resource: https://www.programiz.com/python-programming/exception-handling
# Additional Resource: https://realpython.com/python-exceptions/
# Additional Resource: https://www.w3schools.com/python/python_try_except.asp