# File Handling
"""
File handling in Python refers to the process of reading from and writing to files. Python provides built-in functions and methods to manipulate files, allowing for file operations such as opening, reading, writing, and closing files. The key function for working with files in Python is 'open()', which is used to open a file and returns a file object. Using this file object, you can perform various operations like reading from the file (using methods like read(), readline(), or readlines()) and writing to the file (using methods like write() or writelines()). It is important to close a file after your operations are completed to free up system resources using the close() method or by using the 'with' statement to handle files, which ensures the file is automatically closed.
"""

# Example: # Example 1: Opening and reading from a file
with open('example.txt', 'r') as file:
    content = file.read()
    print(content)
# Example: # Example 2: Writing to a file
with open('example.txt', 'w') as file:
    file.write('Hello, World!')
# Example: # Example 3: Appending to a file
with open('example.txt', 'a') as file:
    file.write('Hello again!')
# Example: # Example 4: Reading lines from a file into a list
with open('example.txt', 'r') as file:
    lines = file.readlines()
    for line in lines:
        print(line.strip())
# Example: # Example 5: Using read and readline
with open('example.txt', 'r') as file:
    line = file.readline()
    while line:
        print(line.strip())
        line = file.readline()
# Exercise: 1. Write a Python script to read a file 'data.txt' and print its contents line by line.
# Exercise: 2. Modify the Python script to count the number of lines in 'data.txt'.
# Exercise: 3. Write a Python program that creates a new file called 'log.txt' and writes the current date and time into it.
# Exercise: 4. Extend the above program to append a new log entry for each time the Python script is run.
# Exercise: 5. Create a Python script that reads a file and prints only those lines that contain the word 'Python'.
# Additional Resource: https://docs.python.org/3/tutorial/inputoutput.html#reading-and-writing-files
# Additional Resource: https://realpython.com/read-write-files-python/
# Additional Resource: https://www.geeksforgeeks.org/file-handling-python/