# Libraries and Frameworks (NumPy, Pandas)
"""
In Python, libraries are collections of modules or functions that are not part of the core language but are bundled together to provide additional functionality. Frameworks, although similar to libraries, provide a more comprehensive set of tools and a structure to develop specific types of applications. NumPy is a library for the Python programming language, adding support for large, multi-dimensional arrays and matrices, along with a large collection of high-level mathematical functions to operate on these arrays. Pandas is a fast, powerful, flexible, and easy-to-use open-source data analysis and manipulation tool built on top of the Python programming language. It provides data structures like Series and DataFrame for handling structured data.
"""

# Example: # Example 1: Importing NumPy and creating an array
import numpy as np
arr = np.array([1, 2, 3, 4, 5])
print(arr)

# Example: # Example 2: Creating a Pandas DataFrame from a dictionary
import pandas as pd
data = {'col1': [1, 2], 'col2': [3, 4]}
df = pd.DataFrame(data)
print(df)

# Example: # Example 3: Using NumPy to perform a mathematical operation on an array
arr = np.array([1, 2, 3, 4, 5])
arr_squared = np.square(arr)
print(arr_squared)

# Example: # Example 4: Selecting a subset of data from a DataFrame using Pandas
import pandas as pd
data = {'name': ['Alice', 'Bob', 'Charlie'], 'age': [25, 30, 35]}
df = pd.DataFrame(data)
sub_df = df[df['age'] > 30]
print(sub_df)

# Exercise: # Exercise 1: Create a NumPy array of 10 random integers between 0 to 50 and compute their mean.
# Exercise: # Exercise 2: Load a CSV file into a Pandas DataFrame and display the first 5 rows.
# Exercise: # Exercise 3: Create a Pandas DataFrame with a column 'numbers' containing integers from 1 to 10 and a column 'squares' which stores the square of each number in 'numbers'.
# Exercise: # Exercise 4: Use NumPy to generate a 5x5 matrix with values from 1 to 25, then use Pandas to label the rows and columns with letters 'A' through 'E'.
# Additional Resource: {'title': "NumPy's official documentation", 'url': 'https://numpy.org/doc/stable/'}
# Additional Resource: {'title': '10 Minutes to Pandas - Official pandas tutorial', 'url': 'https://pandas.pydata.org/docs/user_guide/10min.html'}
# Additional Resource: {'title': 'Data Analysis with Python and Pandas: Tutorial Series', 'url': 'https://www.youtube.com/playlist?list=PL6flErFppaj3wPeS-yLzGQhL6PCj__AI0'}
# Additional Resource: {'title': 'Python NumPy Tutorial for Beginners', 'url': 'https://www.youtube.com/watch?v=QUT1VHiLmmI'}