import os
import json
from openai import OpenAI

# Directory setup
directory = "python_tutoring_book_2"
if not os.path.exists(directory):
    os.makedirs(directory)

# Define Python concepts
python_concepts = [
    "Variables and Basic Data Types",
    "Control Flow (if, else, loops)",
    "Functions and Modules",
    "Exception Handling",
    "Object-Oriented Programming",
    "File Handling",
    "Libraries and Frameworks (NumPy, Pandas)",
    "Advanced Topics (Decorators, Generators)",
    "Data Visualization",
    "Machine Learning with Python"
]

# Load system message
with open("python_book_system_message", "r") as file:
    system_message = file.read()
    

# OpenAI API setup
client = OpenAI(api_key=os.getenv("OPENAI_API_KEY") or "YOUR API KEY HERE")

# Loop over Python concepts
for i, concept in enumerate(python_concepts, start=1):
    print(f"Generating content for {concept}...")
    messages = [
        {"role": "system", "content": json.dumps(system_message)},
        {"role": "user", "content": f"concept you are working on: {concept}"}
    ]

    try:
        response = client.chat.completions.create(
            model="gpt-4-1106-preview",
            response_format={"type": "json_object"},
            stream=True,
            messages=messages
        )

        responses = ''
        # Process and save the response
        # Process and save the response
        for chunk in response:
            if chunk.choices[0].delta.content:
                text_chunk = chunk.choices[0].delta.content
                print(text_chunk, end="", flush=True)  # Print each chunk as it arrives
                responses += str(text_chunk)

        # Parse the final responses string as JSON
        json_responses = json.loads(responses)

        # Extract fields from the JSON object
        concept = json_responses["concept"]
        explanation = json_responses["explanation"]
        examples = json_responses["examples"]
        exercises = json_responses["exercises"]
        additional_resources = json_responses["additional_resources"]

        # Prepare the content to be written to the file
        content = f"# {concept}\n\"\"\"\n{explanation}\n\"\"\"\n"

        # Add each example, exercise, and additional resource as a comment
        for example in examples:
            content += f"\n# Example: {example}"
        for exercise in exercises:
            content += f"\n# Exercise: {exercise}"
        for resource in additional_resources:
            content += f"\n# Additional Resource: {resource}"

        # Save to a file
        print(f"Saving content for {concept}...")        
        file_name = f"{directory}/chapter_{i}_{concept.replace(' ', '_').lower()}.py"
        with open(file_name, 'w') as file:  # 'w' to overwrite each chunk
            file.write(content)

    except Exception as e:
        print(f"Error: {e}")

